    #include "Dbrowser.ch"

    // Creates and returns a TBrowse object for a browse with a while
    // and for condition and append mode

    FUNCTION TBrowseFW(bWhile, bFor, bFirst, bLast)

    LOCAL oTbr  := TBrowseNew()

      oTbr:cargo      := Array(DB_NUM_IVARS)
      oTbr:while      := bWhile
      oTbr:for        := bFor
      oTbr:goFirst    := bFirst
      oTbr:goLast     := bLast
      oTbr:goNext     := {|| TBNext(oTbr) }
      oTbr:goPrev     := {|| TBPrev(oTbr) }
      oTbr:appendMode := .F.

      oTbr:goTopBlock    := {||  TBFirst(oTbr) }
      oTbr:goBottomBlock := {||  TBLast(oTbr)  }
      oTbr:skipBlock     := {|n| DbSkipBlock(n, oTbr) }

      oTbr:goTop()

    RETURN oTbr


    // Routine called by goTopBlock. It must move to the first record
    // in the database that matches both the FOR and WHILE scopes.
    // These are stored as code blocks in the TBrowse object.
    // If the routine does not find a matching record, it must
    // position the record pointer at end of file.

    FUNCTION TBFirst(oTbr)
    
      // This takes us to the first one
      Eval(oTbr:goFirst)
  
      // But we must ensure this also matches the FOR scope,
      // without moving outside the records matching the WHILE scope.
      DO WHILE !Eof() .AND. Eval(oTbr:while) .AND. !Eval(oTbr:for)
        SKIP
      ENDDO

      // If we are not on a valid record, move to end of file.
      IF Eof() .OR. !Eval(oTbr:while)
        GOTO 0
      ENDIF
    
    RETURN NIL


    // Routine called by goBottomBlock to move to the last record
    // in the database matching the for and while condition, and
    // taking append mode into account.

    FUNCTION TBLast(oTbr)
    
      // If append mode is on, move to end of file
      IF oTbr:appendMode
        GOTO 0
      ELSE
        // This takes us to the last record
        Eval(oTbr:goLast)

        // This ensures the record matches both the for and while scope
        DO WHILE !Bof() .AND. Eval(oTbr:while) .AND. !Eval(oTbr:for)
          SKIP -1
        ENDDO
    
        // If we are not on a valid record, go to end of file
        IF Bof() .OR. !Eval(oTbr:while)
          GOTO 0
        ENDIF
      ENDIF

    RETURN NIL


    // Move to next record supporting for and while conditions
    // and append mode. Returns .T. if successful, .F. otherwise
    // If it can't move, it doesn't change the record pointer.

    FUNCTION TBNext(oTbr)

    // First save record pointer so we can restore it
    // if we can't move.
    LOCAL nSaveRecNum := recno()
    LOCAL lMoved := .T.

      IF Eof()
        lMoved := .F.
      ELSE
        SKIP
        DO WHILE !Eval(oTbr:for) .AND. Eval(oTbr:while) .AND. !Eof()
          SKIP
        ENDDO

        IF Eof() .AND. oTbr:appendMode
          // fine ...
        ELSEIF !Eval(oTbr:while) .OR. Eof()
          IF !Eval(oTbr:while) .AND. oTbr:appendMode
            GOTO 0
          ELSE
            lMoved := .F.
            GOTO nSaveRecNum
          ENDIF
        ENDIF
      ENDIF

    RETURN lMoved


    // Move to previous record supporting for and while conditions
    // and append mode

    FUNCTION TBPrev(oTbr)

    LOCAL nSaveRecNum := Recno()
    LOCAL lMoved := .T.

      // Take special care at Eof() - a SKIP -1 is not sufficient
      // because of the filters
      IF Eof()
        Eval(oTbr:goLast)
      ELSE
        SKIP -1
      ENDIF

      DO WHILE !Eval(oTbr:for) .AND. Eval(oTbr:while) .AND. !Bof()
        SKIP -1
      ENDDO

      IF !Eval(oTbr:while) .OR. Bof()
        GOTO nSaveRecNum
        lMoved := .F.
      ENDIF

    RETURN lMoved




    // Generic skipblock for database browses. Needs supporting
    // code blocks that move forward by one record and move backward
    // by one record. They must both return logicals which indicate
    // whether they suceeded or not.
    // This function returns how many records it skipped, as required
    // by TBrowse:skipBlock.

    FUNCTION DbSkipBlock(n, oTbr)

    LOCAL nSkipped := 0

      // Idiosyncrasy for database skipper
      IF n = 0
        SKIP 0
      ELSEIF n > 0
        DO WHILE nSkipped != n .AND. Eval(oTbr:goNext)
          nSkipped++
        ENDDO
      ELSE
        DO WHILE nSkipped != n .AND. Eval(oTbr:goPrev)
          nSkipped--
        ENDDO
      ENDIF

    RETURN nSkipped
